(define-module (alterator ahttpd html)
  :use-module (srfi srfi-1)
  :use-module (srfi srfi-13)

  :use-module (alterator woo)
  :use-module (alterator str)
  :use-module (alterator algo)
  :use-module (alterator gettext)
  
  :use-module (alterator http html)
  :use-module (alterator http template)
  :export (html:hidden
           html:submit
	   html:select-options
           html:select-option-value
           html:select-option-label
	   html:select

	   html:redirect
	   catch/message))

(define (html:hidden name value)
  (html: "input"
         (@ 'name (->string name))
         (@ 'type "hidden")
         (@ 'value value)))

(define (html:submit name value . args)
  (apply html: "input"
         (@ 'name (->string name))
         (@ 'type "submit")
	 (@ 'class "btn")
         (@ 'value value) args))

(define (html:selected current value)
  (if (string=? value current)
      (@ 'selected "true")
      ""))

(define (html:select-option-value x)
  (woo-get-option x 'name))

(define (html:select-option-label x)
  (woo-get-option x 'label (html:select-option-value x)))

(define (html:select-options value variants)
  ;;auto select first item
  (let ((selected (if (and (string-null? value)
                           (pair? variants))
                      (html:select-option-value (car variants))
                      value)))
    (map (lambda(current)
           (let ((label (html:select-option-label current))
                 (value (html:select-option-value current)))
             (html: "option"
                    (@ 'value value)
                    (html:selected selected value)
                    label)))
           variants)))

(define (html:select name value options)
  (html: "select"
         (@ 'name (symbol->string name))
	 ,(html:select-options value variants)))

;;; redirect

(define (html:redirect new-url . cookies)
  (apply throw 'html-redirect-error new-url cookies))

;;; catch

(define (html-error-message msg)
  (html: "div"
	 (@ 'class "error-message")
	 msg))

(define (html-error scm msg)
  (template
    scm
    (tag: "body" (lambda (options content)
		   `(body ,(html-error-message msg)
			  ,@content)))))

(define (document-get-label-by-name scm name)
  (call-with-current-continuation
    (lambda(return)
      (template
	scm
	(tag: "span" (lambda(options content)
		       (let ((n (cond-assoc 'name options))
			     (t (cond-assoc 'translate options "_")))
			 (and (string? n)
			      (string=? n name)
			      (return `(span ,(@ 'translate t) ,@content)))))))
      #f)))

(define (type-error-string scm reasonlist)
  (filter-map (lambda(x)
		(let ((l (document-get-label-by-name scm (car x))))
		  (and l `(,l "  " ,(cdr x) ,(html: "br")))))
	      reasonlist))

(define (woo-error-string reason)
  (html: "pre" reason))

(define (catch/message proc fallback)
  (catch 'type-error
	 (lambda()
	   (catch 'woo-error
		  proc
		  (lambda (key reason)
		    (html-error (fallback) (woo-error-string reason)))))
	 (lambda (key reasonlist)
	   (let ((scm (fallback)))
	     (html-error scm (type-error-string scm reasonlist))))))
